#!/bin/bash
# LTSP over WLAN -script for Ubuntu 8.10
#
# This script will make a initrd.img.ltsp.wlan -file which contains
# drivers for WLAN-chipset and modifications to LTSP boot scripts.
# So you can boot and load vmlinuz and initrd.img.ltsp.wlan from local
# media. If you are lucky then your machine continue booting to LTSP thin 
# client over WLAN.

# Copyright (C) January 2009 Jussi Keranen

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Variables

basedir="/usr/share/initramfs-tools"

init_file="$basedir/init"
export_lines_file="./export_lines"
command_line_options_file="./case_lines"

nbd_file="$basedir/scripts/ltsp_nbd"
set_device_and_wpa_file="./if_wlandevice"
set_serverip_file="./if_serverip"

wpa_file="$basedir/scripts/wpa"
wpa_source_file="./wpa"
wpa_tools_file="$basedir/hooks/wpatools"
wpa_tools_source_file="./wpatools"

wlan_modules_file="$basedir/hooks/wlanmodules"
wlan_modules_source_file="./wlanmodules.conf"

newinitrd_file="initrd.img.ltsp.wlan"

# Probably you don't want to edit lines below this one

function check_ltsp_chroot {
    if [ -d /opt/ltsp ] || [ $UID != 0 ]
    then
    echo -e "\nYou are not chrooted to /opt/ltsp/{arch}.\n"
    echo -e "Please do chroot this way and try again. (example for i386):"
    echo -e "$ sudo mount --bind /dev /opt/ltsp/i386/dev"
    echo -e "$ sudo mount -t proc none /opt/ltsp/i386/proc"
    echo -e "$ sudo chroot /opt/ltsp/i386\n"
    exit
    fi
}

function check_files_exist {

    if [ ! -f $init_file ]                 || \
       [ ! -f $export_lines_file ]         || \
       [ ! -f $command_line_options_file ] || \
       [ ! -f $nbd_file ]                  || \
       [ ! -f $set_device_and_wpa_file ]   || \
       [ ! -f $set_serverip_file ]         || \
       [ ! -f $wpa_source_file ]           || \
       [ ! -f $wpa_tools_source_file ]     || \
       [ ! -f $wlan_modules_source_file ]
    then
        echo -e "\nMake sure you have all of following files:\n"
        echo -e "$init_file"
        echo -e "$nbd_file"
        echo -e "$export_lines_file"
        echo -e "$command_line_options_file"
        echo -e "$set_device_and_wpa_file"
        echo -e "$set_serverip_file"
        echo -e "$wpa_source_file"
        echo -e "$wlan_modules_source_file"
        echo -e "$wpa_tools_source_file"
        echo -e "\n$newinitrd_file IS NOT generated!"
        exit
    fi
    if [ -f $newinitrd_file ]
    then
        echo -e "\nDestination file $newinitrd_file already exists."
        echo -e "\nIt's NOT overwrited."
        exit
    fi
}

function check_wpa_supplicant_installed {
    if [ ! -f /sbin/wpa_supplicant ]
    then
        echo -e "\nWpa_supplicant is not installed.\n"
        echo -e "We will install it now."
        apt-get -q install wpasupplicant
    fi

    if [ ! -f /sbin/wpa_supplicant ]
    then
        echo -e "\nError while installing wpasupplicant"
        echo -e "\nPlease install it manually:"
        echo -e "\n# apt-get install wpasupplicant"
        exit
    fi
}

function check_restricted_modules {
    if [ `cat $wlan_modules_source_file | grep "ath_pci" | head -c1` ];
    then 
        if [ ! `find /lib/modules -name 'ath_pci*'` ];
        then 
            echo -e "We need linux-restricted-modules -package\n"
            echo -e "Let's install it now."
            apt-get -q install linux-restricted-modules
        fi
        if [ ! `find /lib/modules -name 'ath_pci*'` ];
        then
            echo -e "\nCan't install linux-restricted-modules."
            echo -e "\nPlease install it manually and try again."
            exit
        fi
    fi
}

function check_broadcom_firmware {
  if [ `cat $wlan_modules_source_file | grep "b43" | head -c1` ];
  then
    if [ ! `find /lib/firmware -name 'b43' | head -c1` ]
    then
      echo -e "We need firmware-files for broadcom chipset."
      echo -e "\nPress enter to install it from Cafuego repository."
      read;
      if [ ! `cat /etc/apt/sources.list | grep cafuego | head -c1` ] 
      then
        echo -e "Adding repository..."
        echo "deb     http://ubuntu.cafuego.net/ intrepid-cafuego broadcom" >> /etc/apt/sources.list 
        echo "deb-src http://ubuntu.cafuego.net/ intrepid-cafuego broadcom" >> /etc/apt/sources.list
        echo -e "Loading gpg-key..."
        wget -q http://ubuntu.cafuego.net/cafuego.gpg
        echo -e "Adding gpg-key..."
        apt-key add cafuego.gpg
        rm cafuego.gpg
      fi
      echo -e "Updating sources..."
      apt-get -qq update
      echo -e "Installing b43-firmware..."
      apt-get -qq install b43-firmware  
    fi
     if [ ! `find /lib/firmware -name 'b43' | head -c1` ]
     then
       echo -e "\nCan't install the firmware."
       echo -e "\nPlease install it manually and try again."
       echo -e "\nhttp://ubuntu.cafuego.net/dists/intrepid-cafuego/broadcom"
       exit
     fi
  fi
}

function backup_initramfs_settings {
cp -r /usr/share/initramfs-tools /usr/share/initramfs-tools.original
}

function modify_init {
    echo -e "\nModifying init-file..."

    line42=`cat $init_file | sed -n -e '42,$p' | head -n 1`
    line42check="export resume_offset="

    line44=`cat $init_file | sed -n -e '44,$p' | head -n 1`
    line44check="# Bring in the main config"

    line159=`cat $init_file | sed -n -e '159,$p' | head -n 1`
    line159check="done"

    if [ "$line42" == "$line42check" ] && [ "$line44" == "$line44check" ] && [ "$line159" == "$line159check" ]
    then 
        cp $init_file $init_file.original
        head -n 42 $init_file.original          > $init_file
        cat        $export_lines_file           >> $init_file
        sed -n -e '43,$p' $init_file.original   | head -n 115 >> $init_file
        cat        $command_line_options_file   >> $init_file
        sed -n -e '158,$p' $init_file.original  >> $init_file
    fi
}

function modify_nbd {
    echo -e "Modifying nbd-file..."

    line15=`cat $nbd_file | sed -n -e '15,$p' | head -n 1`
    line15check="	modprobe -q af_packet"
    line49=`cat $nbd_file | sed -n -e '49,$p' | head -n 1 | head -c 12`
    line49check="	. /tmp/net-"

    if [ "$line15" == "$line15check" ] && [ "$line49" == "$line49check" ]
    then
        cp $nbd_file $nbd_file.original
        head -n 15 $nbd_file.original           > $nbd_file
        cat        $set_device_and_wpa_file     >> $nbd_file
        sed -n -e '16,$p' $nbd_file.original   | head -n 34 >> $nbd_file
        cat        $set_serverip_file           >> $nbd_file
        sed -n -e '50,$p' $nbd_file.original    >> $nbd_file
    fi
}

function copy_wpa {
    echo -e "Copying wpa-scripts..."
    cp $wpa_source_file $wpa_file
    chmod +x $wpa_file
}

function copy_wpatools {
    cp $wpa_tools_source_file $wpa_tools_file
    chmod +x $wpa_tools_file
}

function copy_wlanmodules {
    echo -e "Copying wlan-modules..."
    cp $wlan_modules_source_file $wlan_modules_file
    chmod +x $wlan_modules_file
}

function update_initram {
    cp /boot/initrd.img /boot/initrd.img.original
    update-initramfs -u
    cp /boot/initrd.img $newinitrd_file
    cp /boot/initrd.img.original /boot/initrd.img 
}

function cleaning {
    echo -e "Cleaning..."
    if [ $PWD != "/usr/share/initramfs-tools" ]
    then
        rm -r /usr/share/initramfs-tools
        mv /usr/share/initramfs-tools.original /usr/share/initramfs-tools
    fi
    rm /boot/initrd.img.original 
}

function message {
    if [ -f "$newinitrd_file" ]
    then
        echo -e "\nFile $newinitrd_file created.\n"
        echo -e "Please copy it to local media of your laptop."
        echo -e "And boot with a bootloader. Here are example parameters:\n"
        echo -e "kernel /vmlinuz wlandevice=wlan0 serverip=192.168.1.1 essid=mywlan passwd=\"PaSsWoRd\""
        echo -e "initrd /$newinitrd_file\n\n"
        echo -e "Thanks for using LTSP_over_WLAN.\nPlease send comments to jussiker at gmail dot com.\n\n"
    fi
}

function generate_other_files {

    if [ ! -f $export_lines_file ]
    then
        cat >$export_lines_file <<-EOF
export wlandevice=
export serverip=
export essid=
export passwd=
EOF
    chmod +x $export_lines_file
    fi

    if [ ! -f $command_line_options_file ]
    then
        cat >$command_line_options_file <<-EOF
        wlandevice=*)
                wlandevice=\${x#wlandevice=}
                ;;
        serverip=*)
                serverip=\${x#serverip=}
                ;;
        essid=*)
                essid=\${x#essid=}
                ;;
        passwd=*)
                passwd=\${x#passwd=}
                ;;
EOF
    chmod +x $command_line_options_file
    fi

    if [ ! -f $set_device_and_wpa_file ]
    then
        cat >$set_device_and_wpa_file <<-EOF

    if [ -n "\$wlandevice" ]; then
        DEVICE=\$wlandevice
    fi
    . /scripts/wpa
EOF
    chmod +x $set_device_and_wpa_file
    fi

    if [ ! -f $set_serverip_file ]
    then
        cat >$set_serverip_file <<-EOF
    
    if [ -n "\$serverip" ]; then
        ROOTSERVER=\$serverip
    fi
EOF
    chmod +x $set_serverip_file
    fi
    
    if [ ! -f $wpa_source_file ]
    then
        cat >$wpa_source_file <<-EOFILE
cat > /etc/wpa_supplicant.conf <<- EOF 
ctrl_interface=/var/run/wpa_supplicant
# Only WPA-PSK is used. Any valid cipher combination is accepted.
network={
        ssid="\$essid"
        proto=WPA
        key_mgmt=WPA-PSK
        pairwise=CCMP TKIP
        group=CCMP TKIP WEP104 WEP40
        psk=\$passwd
        priority=2
}
EOF
sleep 5 
/sbin/wpa_supplicant -B -qq -Dwext -i\$wlandevice -c/etc/wpa_supplicant.conf
sleep 5
EOFILE
    chmod +x $wpa_source_file
    fi

    if [ ! -f $wpa_tools_source_file ]
    then
        cat >$wpa_tools_source_file <<-EOF
#!/bin/sh
# We don't have any prerequirements
case \$1 in
prereqs)
        exit 0
        ;; 
esac 
. /usr/share/initramfs-tools/hook-functions 
copy_exec /sbin/wpa_action /sbin 
copy_exec /sbin/wpa_cli /sbin 
copy_exec /sbin/wpa_supplicant /sbin
EOF
    chmod +x $wpa_tools_source_file
    fi

}
    
# MAIN
echo -e "\nLTSP_over_WLAN ver 0.8 for Ubuntu 8.10\n(C) January 2009 Jussi Keranen\n"
echo -e "Please make sure you have configured wlanmodules.conf with\
 wlmodsconfgen\nor manually."
echo -e "Press enter to start..."; read;
check_ltsp_chroot 
generate_other_files
check_files_exist 
check_wpa_supplicant_installed
check_restricted_modules
check_broadcom_firmware
backup_initramfs_settings
modify_init 
modify_nbd 
copy_wpa 
copy_wpatools 
copy_wlanmodules 
update_initram 
cleaning 
message
